@@ -11,7 +11,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: discovery.c,v 1.18 2002/04/09 17:28:39 dfs Exp $";
+"$Id: discovery.c,v 1.23 2003/02/05 15:14:36 dfs Exp $";
 
 #include "pppoe.h"
 
@@ -42,6 +42,9 @@ static char const RCSID[] =
 
 #include <signal.h>
 
+/* Supplied by pppd if we're a plugin */
+extern int persist;
+
 /**********************************************************************
 *%FUNCTION: parseForHostUniq
 *%ARGUMENTS:
@@ -216,15 +219,18 @@ parsePADSTags(UINT16_t type, UINT16_t len, unsigned char *data,
     case TAG_SERVICE_NAME_ERROR:
 	syslog(LOG_ERR, "PADS: Service-Name-Error: %.*s", (int) len, data);
 	fprintf(stderr, "PADS: Service-Name-Error: %.*s\n", (int) len, data);
-	exit(1);
+	conn->PADSHadError = 1;
+	break;
     case TAG_AC_SYSTEM_ERROR:
 	syslog(LOG_ERR, "PADS: System-Error: %.*s", (int) len, data);
-	fprintf(stderr, "PADS: System-Error: %.*s\n", (int) len, data);
-	exit(1);
-    case TAG_GENERIC_ERROR:
+	fprintf(stderr, "PADS: System-Error: %.*s\n", (int) len, data); 
+	conn->PADSHadError = 1;
+	break;
+   case TAG_GENERIC_ERROR:
 	syslog(LOG_ERR, "PADS: Generic-Error: %.*s", (int) len, data);
 	fprintf(stderr, "PADS: Generic-Error: %.*s\n", (int) len, data);
-	exit(1);
+	conn->PADSHadError = 1;
+	break;
     case TAG_RELAY_SESSION_ID:
 	conn->relayId.type = htons(type);
 	conn->relayId.length = htons(len);
@@ -250,12 +256,14 @@ sendPADI(PPPoEConnection *conn)
     PPPoETag *svc = (PPPoETag *) (&packet.payload);
     UINT16_t namelen = 0;
     UINT16_t plen;
+    int omit_service_name = 0;
 
     if (conn->serviceName) {
 	namelen = (UINT16_t) strlen(conn->serviceName);
+	if (!strcmp(conn->serviceName, "NO-SERVICE-NAME-NON-RFC-COMPLIANT")) {
+	    omit_service_name = 1;
+	}
     }
-    plen = TAG_HDR_SIZE + namelen;
-    CHECK_ROOM(cursor, packet.payload, plen);
 
     /* Set destination to Ethernet broadcast address */
     memset(packet.ethHdr.h_dest, 0xFF, ETH_ALEN);
@@ -267,14 +275,20 @@ sendPADI(PPPoEConnection *conn)
     packet.code = CODE_PADI;
     packet.session = 0;
 
-    svc->type = TAG_SERVICE_NAME;
-    svc->length = htons(namelen);
-    CHECK_ROOM(cursor, packet.payload, namelen+TAG_HDR_SIZE);
+    if (!omit_service_name) {
+	plen = TAG_HDR_SIZE + namelen;
+	CHECK_ROOM(cursor, packet.payload, plen);
 
-    if (conn->serviceName) {
-	memcpy(svc->payload, conn->serviceName, strlen(conn->serviceName));
+	svc->type = TAG_SERVICE_NAME;
+	svc->length = htons(namelen);
+
+	if (conn->serviceName) {
+	    memcpy(svc->payload, conn->serviceName, strlen(conn->serviceName));
+	}
+	cursor += namelen + TAG_HDR_SIZE;
+    } else {
+	plen = 0;
     }
-    cursor += namelen + TAG_HDR_SIZE;
 
     /* If we're using Host-Uniq, copy it over */
     if (conn->useHostUniq) {
@@ -547,16 +561,20 @@ waitForPADS(PPPoEConnection *conn, int timeout)
 	/* Is it PADS?  */
 	if (packet.code == CODE_PADS) {
 	    /* Parse for goodies */
+	    conn->PADSHadError = 0;
 	    parsePacket(&packet, parsePADSTags, conn);
-	    conn->discoveryState = STATE_SESSION;
-	    break;
+	    if (!conn->PADSHadError) {
+		conn->discoveryState = STATE_SESSION;
+		break;
+	    }
 	}
     } while (conn->discoveryState != STATE_SESSION);
 
     /* Don't bother with ntohs; we'll just end up converting it back... */
     conn->session = packet.session;
 
-    syslog(LOG_INFO, "PPP session is %d", (int) ntohs(conn->session));
+    syslog(LOG_INFO, "PPP session is %d (0x%x)", (int) ntohs(conn->session),
+	   (unsigned int) ntohs(conn->session));
 
     /* RFC 2516 says session id MUST NOT be zero or 0xFFFF */
     if (ntohs(conn->session) == 0 || ntohs(conn->session) == 0xFFFF) {
@@ -578,7 +596,7 @@ discovery(PPPoEConnection *conn)
 {
     int padiAttempts = 0;
     int padrAttempts = 0;
-    int timeout = PADI_TIMEOUT;
+    int timeout = conn->discoveryTimeout;
 
     /* Skip discovery and don't open discovery socket? */
     if (conn->skipDiscovery && conn->noDiscoverySocket) {
@@ -602,7 +620,13 @@ discovery(PPPoEConnection *conn)
     do {
 	padiAttempts++;
 	if (padiAttempts > MAX_PADI_ATTEMPTS) {
-	    rp_fatal("Timeout waiting for PADO packets");
+	    if (persist) {
+		padiAttempts = 0;
+		timeout = conn->discoveryTimeout;
+		printErr("Timeout waiting for PADO packets");
+	    } else {
+		rp_fatal("Timeout waiting for PADO packets");
+	    }
 	}
 	sendPADI(conn);
 	conn->discoveryState = STATE_SENT_PADI;
@@ -624,11 +648,17 @@ discovery(PPPoEConnection *conn)
 	exit(0);
     }
 
-    timeout = PADI_TIMEOUT;
+    timeout = conn->discoveryTimeout;
     do {
 	padrAttempts++;
 	if (padrAttempts > MAX_PADI_ATTEMPTS) {
-	    rp_fatal("Timeout waiting for PADS packets");
+	    if (persist) {
+		padrAttempts = 0;
+		timeout = conn->discoveryTimeout;
+		printErr("Timeout waiting for PADS packets");
+	    } else {
+		rp_fatal("Timeout waiting for PADS packets");
+	    }
 	}
 	sendPADR(conn);
 	conn->discoveryState = STATE_SENT_PADR;
