@@ -16,7 +16,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: if.c,v 1.16 2002/07/08 14:28:40 dfs Exp $";
+"$Id: if.c,v 1.17 2005/08/08 03:51:35 dfs Exp $";
 
 #include "pppoe.h"
 
@@ -533,7 +533,7 @@ sendPacket(PPPoEConnection *conn, int sock, PPPoEPacket *pkt, int size)
 	return -1;
     }
 #elif defined(HAVE_STRUCT_SOCKADDR_LL)
-    if (send(sock, pkt, size, 0) < 0) {
+    if (send(sock, pkt, size, 0) < 0 && (errno != ENOBUFS)) {
 	sysErr("send (sendPacket)");
 	return -1;
     }
