@@ -9,14 +9,14 @@
 * This program may be distributed according to the terms of the GNU
 * General Public License, version 2 or (at your option) any later version.
 *
-* $Id: pppoe-server.c,v 1.87 2002/06/13 14:20:47 dfs Exp $
+* $Id: pppoe-server.c,v 1.91 2005/08/10 00:25:21 dfs Exp $
 *
 * LIC: GPL
 *
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: pppoe-server.c,v 1.87 2002/06/13 14:20:47 dfs Exp $";
+"$Id: pppoe-server.c,v 1.91 2005/08/10 00:25:21 dfs Exp $";
 
 #include "config.h"
 
@@ -1014,7 +1014,7 @@ usage(char const *argv0)
 #endif
 
     fprintf(stderr, "   -h             -- Print usage information.\n\n");
-    fprintf(stderr, "PPPoE-Server Version %s, Copyright (C) 2001 Roaring Penguin Software Inc.\n", VERSION);
+    fprintf(stderr, "PPPoE-Server Version %s, Copyright (C) 2001-2005 Roaring Penguin Software Inc.\n", VERSION);
 
 #ifndef HAVE_LICENSE
     fprintf(stderr, "PPPoE-Server comes with ABSOLUTELY NO WARRANTY.\n");
@@ -1056,6 +1056,12 @@ main(int argc, char **argv)
     char *options = "hI:C:L:R:T:m:FN:f:o:skp:lrudPc:S:1";
 #endif
 
+    if (getuid() != geteuid() ||
+	getgid() != getegid()) {
+	fprintf(stderr, "SECURITY WARNING: pppoe-server will NOT run suid or sgid.  Fix your installation.\n");
+	exit(1);
+    }
+
     memset(interfaces, 0, sizeof(interfaces));
 
     /* Initialize syslog */
@@ -1632,8 +1638,7 @@ startPPPDUserMode(ClientSession *session)
 	     ntohs(session->sess),
 	     session->eth[0], session->eth[1], session->eth[2],
 	     session->eth[3], session->eth[4], session->eth[5],
-	     session->serviceName,
-	     PppoeOptions);
+	     PppoeOptions, session->serviceName);
     argv[c++] = strdup(buffer);
     if (!argv[c-1]) {
 	/* TODO: Send a PADT */
@@ -2036,7 +2041,7 @@ sendHURLorMOTM(PPPoEConnection *conn, char const *url, UINT16_t tag)
 
     hurl.type = htons(tag);
     hurl.length = htons(elen);
-    strcpy(hurl.payload, url);
+    strcpy((char *) hurl.payload, url);
     memcpy(cursor, &hurl, elen + TAG_HDR_SIZE);
     cursor += elen + TAG_HDR_SIZE;
     plen += elen + TAG_HDR_SIZE;
