@@ -4,7 +4,7 @@
 *
 * Implementation of user-space PPPoE redirector for Linux.
 *
-* Copyright (C) 2000-2001 by Roaring Penguin Software Inc.
+* Copyright (C) 2000-2005 by Roaring Penguin Software Inc.
 *
 * This program may be distributed according to the terms of the GNU
 * General Public License, version 2 or (at your option) any later version.
@@ -14,7 +14,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: pppoe.c,v 1.35 2002/05/08 13:43:06 dfs Exp $";
+"$Id: pppoe.c,v 1.41 2005/08/10 00:25:21 dfs Exp $";
 
 #include "pppoe.h"
 
@@ -69,6 +69,8 @@ int optFloodDiscovery    = 0;   /* Flood server with discovery requests.
 
 PPPoEConnection *Connection = NULL; /* Must be global -- used
 				       in signal handler */
+
+int persist = 0; 		/* We are not a pppd plugin */
 /***********************************************************************
 *%FUNCTION: sendSessionPacket
 *%ARGUMENTS:
@@ -230,6 +232,9 @@ session(PPPoEConnection *conn)
     /* Open a session socket */
     conn->sessionSocket = openInterface(conn->ifName, Eth_PPPOE_Session, conn->myEth);
 
+    /* Drop privileges */
+    dropPrivs();
+
     /* Prepare for select() */
     if (conn->sessionSocket > maxFD)   maxFD = conn->sessionSocket;
     if (conn->discoverySocket > maxFD) maxFD = conn->discoverySocket;
@@ -356,6 +361,7 @@ usage(char const *argv0)
 	    DEFAULT_IF);
 #endif
     fprintf(stderr, "   -T timeout     -- Specify inactivity timeout in seconds.\n");
+    fprintf(stderr, "   -t timeout     -- Initial timeout for discovery packets in seconds\n");
     fprintf(stderr, "   -D filename    -- Log debugging information in filename.\n");
     fprintf(stderr, "   -V             -- Print version and exit.\n");
     fprintf(stderr, "   -A             -- Print access concentrator names and exit.\n");
@@ -371,7 +377,7 @@ usage(char const *argv0)
     fprintf(stderr, "   -d             -- Perform discovery, print session info and exit.\n");
     fprintf(stderr, "   -f disc:sess   -- Set Ethernet frame types (hex).\n");
     fprintf(stderr, "   -h             -- Print usage information.\n\n");
-    fprintf(stderr, "PPPoE Version %s, Copyright (C) 2001 Roaring Penguin Software Inc.\n", VERSION);
+    fprintf(stderr, "PPPoE Version %s, Copyright (C) 2001-2005 Roaring Penguin Software Inc.\n", VERSION);
     fprintf(stderr, "PPPoE comes with ABSOLUTELY NO WARRANTY.\n");
     fprintf(stderr, "This is free software, and you are welcome to redistribute it under the terms\n");
     fprintf(stderr, "of the GNU General Public License, version 2 or any later version.\n");
@@ -405,10 +411,16 @@ main(int argc, char *argv[])
     long flags;
 #endif
 
+    if (getuid() != geteuid() ||
+	getgid() != getegid()) {
+	IsSetID = 1;
+    }
+
     /* Initialize connection info */
     memset(&conn, 0, sizeof(conn));
     conn.discoverySocket = -1;
     conn.sessionSocket = -1;
+    conn.discoveryTimeout = PADI_TIMEOUT;
 
     /* For signal handler */
     Connection = &conn;
@@ -416,8 +428,17 @@ main(int argc, char *argv[])
     /* Initialize syslog */
     openlog("pppoe", LOG_PID, LOG_DAEMON);
 
-    while((opt = getopt(argc, argv, "I:VAT:D:hS:C:Usm:np:e:kdf:F:")) != -1) {
+    while((opt = getopt(argc, argv, "I:VAT:D:hS:C:Usm:np:e:kdf:F:t:")) != -1) {
 	switch(opt) {
+	case 't':
+	    if (sscanf(optarg, "%d", &conn.discoveryTimeout) != 1) {
+		fprintf(stderr, "Illegal argument to -t: Should be -t timeout\n");
+		exit(EXIT_FAILURE);
+	    }
+	    if (conn.discoveryTimeout < 1) {
+		conn.discoveryTimeout = 1;
+	    }
+	    break;
 	case 'F':
 	    if (sscanf(optarg, "%d", &optFloodDiscovery) != 1) {
 		fprintf(stderr, "Illegal argument to -F: Should be -F numFloods\n");
@@ -474,11 +495,13 @@ main(int argc, char *argv[])
 	    break;
 
 	case 'p':
+	    switchToRealID();
 	    pidfile = fopen(optarg, "w");
 	    if (pidfile) {
 		fprintf(pidfile, "%lu\n", (unsigned long) getpid());
 		fclose(pidfile);
 	    }
+	    switchToEffectiveID();
 	    break;
 	case 'S':
 	    SET_STRING(conn.serviceName, optarg);
@@ -493,7 +516,9 @@ main(int argc, char *argv[])
 	    conn.useHostUniq = 1;
 	    break;
 	case 'D':
+	    switchToRealID();
 	    conn.debugFile = fopen(optarg, "w");
+	    switchToEffectiveID();
 	    if (!conn.debugFile) {
 		fprintf(stderr, "Could not open %s: %s\n",
 			optarg, strerror(errno));
@@ -546,11 +571,7 @@ main(int argc, char *argv[])
 #endif
     }
 
-    /* Set signal handlers: send PADT on HUP; ignore TERM and INT */
     if (!conn.printACNames) {
-	signal(SIGTERM, SIG_IGN);
-	signal(SIGINT, SIG_IGN);
-	signal(SIGHUP, sigPADT);
 
 #ifdef HAVE_N_HDLC
 	if (conn.synchronous) {
@@ -600,6 +621,11 @@ main(int argc, char *argv[])
 	       conn.peerEth[5]);
 	exit(EXIT_SUCCESS);
     }
+
+    /* Set signal handlers: send PADT on HUP; ignore TERM and INT */
+    signal(SIGTERM, SIG_IGN);
+    signal(SIGINT, SIG_IGN);
+    signal(SIGHUP, sigPADT);
     session(&conn);
     return 0;
 }
